@@ -3,6 +3,7 @@
 
 from .diff import CiscoIOSDiffConfig
 from .config import CiscoIOSConfig
+from .utils import VRF_GLOBAL
 
 
-__all__ = [CiscoIOSDiffConfig, CiscoIOSConfig]
\ No newline at end of file
+__all__ = [CiscoIOSDiffConfig, CiscoIOSConfig, VRF_GLOBAL]
\ No newline at end of file
