@@ -1,4 +1,4 @@
-# adimtote.ios.commands.interface
+# asimtote.ios.commands.interface
 #
 # Copyright (C) Robert Franklin <rcf34@cam.ac.uk>
 
@@ -11,7 +11,7 @@
 from deepops import deepsetdefault
 
 from ..utils import interface_canonicalize, expand_set
-from ...config import IndentedContextualCommand
+from ...command import IndentedContextualCommand
 
 
 
@@ -56,6 +56,18 @@ class Cmd_Int_ARPTime(CmdContext_Int):
         cfg["arp-timeout"] = int(time)
 
 
+class Cmd_Int_BFDIntvl(CmdContext_Int):
+    match = (r"bfd interval (?P<intvl>\d+) min_rx (?P<min_rx>\d+)"
+             r" multiplier (?P<mult>\d+)")
+
+    def parse(self, cfg, intvl, min_rx, mult):
+        cfg["bfd-interval"] = {
+            "interval": int(intvl),
+            "min-rx": int(min_rx),
+            "multiplier": int(mult)
+        }
+
+
 class Cmd_Int_CDPEna(CmdContext_Int):
     match = r"(?P<no>no )?cdp enable"
 
@@ -89,6 +101,13 @@ class Cmd_Int_Encap(CmdContext_Int):
         cfg["encapsulation"] = encap.lower()
 
 
+class Cmd_Int_FEC(CmdContext_Int):
+    match = r"fec (?P<fec>.+)"
+
+    def parse(self, cfg, fec):
+        cfg["fec"] = fec
+
+
 class Cmd_Int_IPAccGrp(CmdContext_Int):
     match = r"ip access-group (?P<acl_name>\S+) (?P<dir_>in|out)"
 
@@ -96,7 +115,11 @@ class Cmd_Int_IPAccGrp(CmdContext_Int):
         cfg.setdefault("ip-access-group", {})[dir_] = acl_name
 
 
+
+# =============================================================================
 # ip address ...
+# =============================================================================
+
 
 
 class Cmd_Int_IPAddr(CmdContext_Int):
@@ -116,7 +139,11 @@ class Cmd_Int_IPAddrSec(CmdContext_Int):
         cfg.setdefault("ip-address-secondary", set()).add(addr)
 
 
+
+# =============================================================================
 # ...
+# =============================================================================
+
 
 
 class Cmd_Int_IPFlowMon(CmdContext_Int):
@@ -158,7 +185,11 @@ class Cmd_Int_IPMcastBdry(CmdContext_Int):
         cfg["ip-multicast-boundary"] = acl
 
 
+
+# =============================================================================
 # ip ospf ...
+# =============================================================================
+
 
 
 class Cmd_Int_IPOSPFArea(CmdContext_Int):
@@ -215,7 +246,11 @@ class Cmd_Int_IPOSPFNet(CmdContext_Int):
         cfg.setdefault("ip-ospf", {})["network"] = net
 
 
+
+# =============================================================================
 # ip pim ...
+# =============================================================================
+
 
 
 class Cmd_Int_IPPIMMode(CmdContext_Int):
@@ -232,7 +267,11 @@ class Cmd_Int_IPPIMBSRBdr(CmdContext_Int):
         cfg.setdefault("ip-pim", {})["bsr-border"] = True
 
 
+
+# =============================================================================
 # ...
+# =============================================================================
+
 
 
 class Cmd_Int_IPPolicyRtMap(CmdContext_Int):
@@ -257,7 +296,11 @@ class Cmd_Int_IPVerifyUni(CmdContext_Int):
         cfg["ip-verify-unicast"] = opt
 
 
+
+# =============================================================================
 # ipv6 ...
+# =============================================================================
+
 
 
 class Cmd_Int_IPv6Addr(CmdContext_Int):
@@ -367,7 +410,11 @@ class Cmd_Int_IPv6VerifyUni(CmdContext_Int):
         cfg["ipv6-verify-unicast"] = opt
 
 
+
+# =============================================================================
 # ...
+# =============================================================================
+
 
 
 class Cmd_Int_MPLSIP(CmdContext_Int):
@@ -384,7 +431,11 @@ class Cmd_Int_MTU(CmdContext_Int):
         cfg["mtu"] = int(size)
 
 
+
+# =============================================================================
 # ospfv3 ...
+# =============================================================================
+
 
 
 class Cmd_Int_OSPFv3Area(CmdContext_Int):
@@ -425,7 +476,11 @@ class Cmd_Int_OSPFv3Net(CmdContext_Int):
         cfg.setdefault("ospfv3", {})["network"] = net
 
 
+
+# =============================================================================
 # ...
+# =============================================================================
+
 
 
 class Cmd_Int_ServPol(CmdContext_Int):
@@ -443,7 +498,11 @@ class Cmd_Int_Shutdown(CmdContext_Int):
         cfg["shutdown"] = not no
 
 
+
+# =============================================================================
 # standby ...
+# =============================================================================
+
 
 
 class Cmd_Int_StandbyIP(CmdContext_Int):
@@ -512,7 +571,11 @@ class Cmd_Int_StandbyVer(CmdContext_Int):
         deepsetdefault(cfg, "standby")["version"] = int(ver)
 
 
+
+# =============================================================================
 # ...
+# =============================================================================
+
 
 
 class Cmd_Int_StormCtrl(CmdContext_Int):
@@ -523,7 +586,11 @@ class Cmd_Int_StormCtrl(CmdContext_Int):
         deepsetdefault(cfg, "storm-control")[traffic] = float(level)
 
 
+
+# =============================================================================
 # switchport ...
+# =============================================================================
+
 
 
 class Cmd_Int_SwPort(CmdContext_Int):
@@ -577,7 +644,11 @@ class Cmd_Int_SwPortTrkAlw(CmdContext_Int):
             cfg["switchport-trunk-allow"] = expand_set(vlans)
 
 
+
+# =============================================================================
 # ...
+# =============================================================================
+
 
 
 class Cmd_Int_PcMinLinks(CmdContext_Int):
@@ -587,6 +658,13 @@ class Cmd_Int_PcMinLinks(CmdContext_Int):
         cfg["port-channel-min-links"] = int(links)
 
 
+class Cmd_Int_PcStandaloneDis(CmdContext_Int):
+    match = r"(?P<no>no )?port-channel standalone-disable"
+
+    def parse(self, cfg, no):
+        cfg["standalone-disable"] = not no
+
+
 class Cmd_Int_VRFFwd(CmdContext_Int):
     match = r"vrf forwarding (?P<name>\S+)"
 
