@@ -11,7 +11,7 @@
 from deepops import deepsetdefault
 
 from ..utils import ip_acl_ext_rule_canonicalize, ipv6_acl_rule_canonicalize
-from ...config import IndentedContextualCommand
+from ...command import IndentedContextualCommand
 
 
 
@@ -19,7 +19,9 @@ from ...config import IndentedContextualCommand
 
 
 
+# =============================================================================
 # ip access-list standard ...
+# =============================================================================
 
 
 
@@ -59,7 +61,9 @@ class Cmd_IPACL_Std_Rule(IndentedContextualCommand):
 
 
 
+# =============================================================================
 # ip access-list extended ...
+# =============================================================================
 
 
 
@@ -92,7 +96,9 @@ class Cmd_IPACL_Ext_Rule(IndentedContextualCommand):
 
 
 
+# =============================================================================
 # ipv6 access-list ...
+# =============================================================================
 
 
 
@@ -117,7 +123,9 @@ class Cmd_IPv6ACL_Rule(IndentedContextualCommand):
 
 
 
+# =============================================================================
 # ip as-path access-list ...
+# =============================================================================
 
 
 
@@ -133,7 +141,9 @@ class Cmd_IPASPathACL(IndentedContextualCommand):
 
 
 
+# =============================================================================
 # ip[v6] prefix-list ...
+# =============================================================================
 
 
 
