@@ -21,7 +21,7 @@ from ...misc import get_all_subclasses
 
 
 
-# the converters are all subclasses of DiffConvert which have the 'cmd'
+# the converters are all subclasses of Convert which have the 'cmd'
 # attribute defined, stored as a set - the order is not important as we
 # sort the list later
 #
@@ -29,4 +29,4 @@ from ...misc import get_all_subclasses
 # converter classes
 
 converters = {
-    c for c in get_all_subclasses(DiffConvert) if c.cmd is not None  }
+    c for c in get_all_subclasses(Convert) if c.cmd is not None  }
