@@ -21,8 +21,6 @@ import re
 
 from netaddr import IPNetwork
 
-from ..config import IndentedContextualCommand, IndentedContextualConfig
-
 
 
 # --- constants ---
@@ -43,29 +41,79 @@ VRF_GLOBAL = "_global"
 
 
 
-# interface names and their canonical representations - the canonical
-# versions don't have to be how Cisco IOS always displays them, in any
-# particular context, but just so they can consistently matched against
+# dictionary of interface types - the key is the canonical form for the
+# type (what will be stored in the configuration dictionary), and the
+# value is the full name (an interface name in the parsed configuration
+# can use any leading subset of characters from this)
+#
+# the canonical versions don't have to be how Cisco IOS always displays
+# them, in any particular context, but just so they can consistently
+# matched against, although they do get used when generating difference
+# commands, so have to recognised as the correct type
+#
+# even if the canonical form is the same as the full name, an interface
+# must be listed in here to do a partial match on it during
+# canonicalisation (e.g. to match 'mg' and 'mgmt', it must be listed)
+#
+# note that this dictionary is not used directly, but a processed
+# version _INTERFACE_TYPE_CANONICALS_LOWER, below, is instead)
 
-_INTERFACE_CANONICALS = {
-    "ethernet": "Eth",
-    "fastethernet": "Fa",
-    "fortygigabitethernet": "Fo",
-    "gigabitethernet": "Gi",
-    "hundredgigethernet": "Hu",
-    "loopback": "Lo",
-    "port-channel": "Po",
-    "tengigabitethernet": "Te",
-    "twentyfivegigethernet": "Twe",
-    "vlan": "Vl",
+_INTERFACE_TYPES_CANONICAL = {
+    "Eth": "Ethernet",
+    "Fa": "FastEthernet",
+    "Fo": "FortyGigabitEthernet",
+    "Gi": "GigabitEthernet",
+    "Hu": "HundredGigE",
+    "Lo": "Loopback",
+    "mgmt": "mgmt",
+    "Po": "Port-channel",
+    "Te": "TenGigabitEthernet",
+    "Twe": "TwentyFiveGigE",
+    "Vl": "Vlan",
+}
+
+
+# version of _INTERFACE_TYPE_CANONICALS with the full names lower-cased
+# for simpler matching
+#
+# note that the canonical names are NOT lower-cased
+
+_INTERFACE_TYPES_CANONICAL_LOWER = {
+    canonical: full.lower()
+        for canonical, full in _INTERFACE_TYPES_CANONICAL.items()
 }
 
 
 
-# regular expression for matching interface names and separating the
-# time (e.g. 'Ethernet') from the number (e.g. '1' or '1/1.1')
+def _interface_type_canonicalize(type_):
+    """Internal function to convert the 'type' of an interface (the part
+    excluding the slot/module/port number) into it's canonical form, for
+    consistent matching.
 
-_INTERFACE_NAME_RE = re.compile(r"([-a-z]+)([0-9/.]*)$")
+    The supplied type can be any leading subset of the full name in
+    _INTERFACE_TYPE_CANONICALS_LOWER.
+    """
+
+    # we'll use this a few times, so get it done here
+    type_lower = type_.lower()
+
+    # work through the list of interfaces, trying to find one where the
+    # full name begins with the type supplied here
+    for canonical, full in _INTERFACE_TYPES_CANONICAL_LOWER.items():
+        if full.startswith(type_lower):
+            # we've found one, so return the canonical form of the type
+            return canonical
+
+    # we didn't find one, so return the original type
+    return type_
+
+
+
+# regular expression for matching interface names and separating the
+# time (e.g. 'Ethernet') from the slot/module/port number part (e.g. '1'
+# or '1/1.1')
+
+_INTERFACE_NAME_RE = re.compile(r"([-A-Za-z]+)([0-9/.]*)$")
 
 
 
@@ -73,19 +121,48 @@ def interface_canonicalize(name):
     """This function takes an interface name (including type and
     number) and returns a canonical version of it, handling
     abbreviations or case differences.
-
-    At the moment, this just handles a defined set of names but it
-    should really be more general and handle arbitrary short versions.
     """
 
-    match = _INTERFACE_NAME_RE.match(name.lower())
+    # try to parse the interface 'type' and 'slot/module/number' parts
+    match = _INTERFACE_NAME_RE.match(name)
 
-    if match:
-        type_, num = match.groups()
-        type_ = _INTERFACE_CANONICALS.get(type_, type_).capitalize()
-        name = type_ + num
+    if not match:
+        raise ValueError("interface: %s cannot parse into type and number" % name)
 
-    return name
+    type_, num = match.groups()
+
+    # return the full name with the type part canonicalised
+    return _interface_type_canonicalize(type_) + num
+
+
+
+# set of interface types which represent physical interfaces rather than
+# virtual interfaces
+
+_INTERFACE_TYPES_PHYSICAL = {
+    "Eth", "Fa", "Fo", "Gi", "Hu", "mgmt", "Te", "Twe"
+}
+
+
+
+def is_int_physical(name):
+    """This function takes an interface name and returns True iff the
+    type of that interface is physical (i.e. something like 'Ethernet'
+    rather than 'Vlan'), and it's not a subinterface (with '.' in the
+    slot/module/port number).
+    """
+
+    # try to parse the interface 'type' and 'slot/module/number' parts
+    match = _INTERFACE_NAME_RE.match(name)
+
+    if not match:
+        raise ValueError("interface: %s cannot parse into type and number" % name)
+
+    type_, num = match.groups()
+
+    # see if the type of the interface is a physical type and there is
+    # not a '.' in the number (which would indicate a subinterface)
+    return (type_ in _INTERFACE_TYPES_PHYSICAL) and ('.' not in num)
 
 
 
@@ -107,7 +184,7 @@ def _ipnet4_to_ios(ipnet4):
     if ipnet4.prefixlen == 32:
         return str(ipnet4.ip)
 
-    return str(ipnet4.ip) + " " + str(ipnet4.hostmask)
+    return str(ipnet4.ip) + ' ' + str(ipnet4.hostmask)
 
 
 
@@ -157,7 +234,7 @@ def _ip_acl_std_rule_parse(rule):
         ipnet4 = IPNetwork(match.group("host"))
 
     else:
-        ipnet4 = IPNetwork(match.group("net") + "/" + match.group("mask"))
+        ipnet4 = IPNetwork(match.group("net") + '/' + match.group("mask"))
 
 
     return match.group("action"), ipnet4
@@ -195,6 +272,12 @@ def ip_acl_std_canonicalize(l):
     these are built, sorting each block into address order, before
     adding the sorted rules in the block into the resulting list.
 
+    Note that this can result in 'permit' and 'deny' entries swapping
+    order, as long as the addresses don't overlap.  For example, 'permit
+    host 20.0.0.1', 'deny host 10.0.0.1' would be reversed into 'deny
+    host 10.0.0.1', 'permit host 20.0.0.1' as the rules are organised
+    into addresses order ('10.0.0.1' < '20.0.0.1').
+
     The result is lists which are not necessarily in the same order as
     they were constructed, nor how IOS stores them, but two lists should
     at least be in the same order so they can be directly compared.
@@ -241,7 +324,7 @@ def ip_acl_std_canonicalize(l):
 
     # convert the rules back into IOS text format and return them as a
     # list
-    return [ (action + " " + _ipnet4_to_ios(net)) for action, net in result ]
+    return [ (action + ' ' + _ipnet4_to_ios(net)) for action, net in result ]
 
 
 
@@ -384,9 +467,9 @@ def ip_acl_ext_rule_canonicalize(rule):
         src_port = (
             " %s %s" % (
                 groups["src_port_listop"],
-                " ".join([str(_port_canonicalize(s))
+                ' '.join([str(_port_canonicalize(s))
                               for s
-                              in groups["src_port_list"].split(" ")])))
+                              in groups["src_port_list"].split(' ')])))
 
     elif groups["src_port_1op"]:
         src_port = " %s %d" % (
@@ -405,9 +488,9 @@ def ip_acl_ext_rule_canonicalize(rule):
         dst_port = (
             " %s %s" % (
                 groups["dst_port_listop"],
-                " ".join([str(_port_canonicalize(s))
+                ' '.join([str(_port_canonicalize(s))
                               for s
-                              in groups["dst_port_list"].split(" ")])))
+                              in groups["dst_port_list"].split(' ')])))
 
     elif groups["dst_port_1op"]:
         dst_port = " %s %d" % (
@@ -421,7 +504,7 @@ def ip_acl_ext_rule_canonicalize(rule):
 
     # the destination port could also be an ICMP message type
     elif groups["icmp_type"]:
-        dst_port = " " + groups["icmp_type"]
+        dst_port = ' ' + groups["icmp_type"]
 
 
     established = groups["established"] or ""
@@ -431,9 +514,9 @@ def ip_acl_ext_rule_canonicalize(rule):
     log = groups["log"] or ""
 
 
-    return (action + " " + protocol
-            + " " + src_addr + src_port
-            + " " + dst_addr + dst_port + established + qos + log)
+    return (action + ' ' + protocol
+            + ' ' + src_addr + src_port
+            + ' ' + dst_addr + dst_port + established + qos + log)
 
 
 
@@ -549,7 +632,7 @@ def ipv6_acl_rule_canonicalize(rule):
 
 
     elif groups["icmp_type"]:
-        dst_port = " " + groups["icmp_type"]
+        dst_port = ' ' + groups["icmp_type"]
 
 
     established = groups["established"] or ""
@@ -557,9 +640,9 @@ def ipv6_acl_rule_canonicalize(rule):
     log = groups["log"] or ""
 
 
-    return (action + " " + protocol
-            + " " + src_addr + src_port
-            + " " + dst_addr + dst_port + established)
+    return (action + ' ' + protocol
+            + ' ' + src_addr + src_port
+            + ' ' + dst_addr + dst_port + established)
 
 
 
@@ -661,7 +744,7 @@ def explain_diffs(old, new, indent="", prefix="", to_str_func=None):
     for line in _differ.compare(old, new):
         op, text = line[0], line[2:]
 
-        if op == " ":
+        if op == ' ':
             # line is the same
 
             # if we were previously adding lines, indicate we're not
@@ -673,7 +756,7 @@ def explain_diffs(old, new, indent="", prefix="", to_str_func=None):
             # add this line
             r.append(indent + prefix + text)
 
-        elif op == "+":
+        elif op == '+':
             # line is being added
 
             # if we're not already in a block of added lines, add a
@@ -685,7 +768,7 @@ def explain_diffs(old, new, indent="", prefix="", to_str_func=None):
             # add this line
             r.append(indent + prefix + text)
 
-        elif op == "-":
+        elif op == '-':
             # line is being removed
 
             # doesn't matter if we were already in a block, adding
@@ -697,32 +780,3 @@ def explain_diffs(old, new, indent="", prefix="", to_str_func=None):
 
     # return the resulting list of lines
     return r
-
-
-
-# --- other functions ---
-
-
-
-# regular expression for matching physical interface names; this assumes
-# the interface name as already been through interface_canonicalize()
-#
-# note that it will only match interfaces of the form
-# "<type><num>[/<num2>[/<num3>[/...]]]", deliberately excluding
-# subinterfaces, where the final number with have a dot before it
-
-_INTERFACE_PHYSICAL_RE = (
-    re.compile("^(Eth|Fa|Fo|Gi|Hu|Mgmt|Te|Twe)\d+(/\d+)*$"))
-
-
-def is_int_physical(i):
-    """This function returns if a particular interface is physical or
-    not.  The names it matches are those that the are used as the
-    canonical ones in the parser.
-
-    The purpose of this function is primarily to work out if an
-    interface can be deleted with "no interface ..." so excludes
-    subinterfaces of physical interfaces.
-    """
-
-    return _INTERFACE_PHYSICAL_RE.match(i)
