@@ -19,17 +19,17 @@ from functools import reduce
 
 
 def get_all_subclasses(c):
-    """Return all subclasses of the specified class.
-
-    The returned list will be in depth-first order but otherwise
-    unsorted (e.g. A, then X(A), then Y(X) then Z(A) then B).
+    """Return all subclasses of the specified class as a set, including
+    the subclasses of classes, recursively.
     """
 
-    l = []
-    for s in c.__subclasses__():
-        l.append(s)
-        l.extend(get_all_subclasses(s))
-    return l
+    s = set()
+
+    for cs in c.__subclasses__():
+        s.add(cs)
+        s.update(get_all_subclasses(cs))
+
+    return s
 
 
 
